<?php
/**
 * @package n3t Fields
 * @author Pavel Poles - n3t.cz
 * @copyright (C) 2017 - Pavel Poles - n3t.cz
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
**/

defined('_JEXEC') or die;

use Joomla\Registry\Registry;

class plgSystemN3tFields extends JPlugin {

  protected $autoloadLanguage = true;

  protected function getTemplates()
  {
    static $templates = null;

    if ($templates === null) {
      $db = JFactory::getDbo();
      $query = $db->getQuery(true)
        ->select($db->quoteName('a.name'))
        ->from($db->quoteName('#__extensions', 'a'))
        ->where($db->quoteName('a.client_id') . ' = 0')
        ->where($db->quoteName('a.enabled') . ' = 1')
        ->where($db->quoteName('a.type') . ' = ' . $db->quote('template'))
        ->order($db->quoteName('a.name') . ' ASC');
      $db->setQuery($query);
      $templates = $db->loadColumn();
    }

    return $templates;
  }

  protected function getFiles($name)
  {
    $files = array();
    $file = str_replace('.', '/', $name) . '.xml';

    foreach ($this->getTemplates() as $template) {
      if (JFile::exists(JPATH_SITE . '/templates/' . $template . '/n3tfields/' . $file))
        $files[$template] = JPATH_SITE . '/templates/' . $template . '/n3tfields/' . $file;
    }

    return $files;
  }

  protected function getContext($context)
  {
    if (strpos($context, 'com_categories.category') === 0)
      $context = JFactory::getApplication()->input->getCmd('extension') . '.category';
    else if (in_array($context, array('com_users.profile', 'com_users.user', 'com_admin.profile')))
      $context = 'com_users.user';
    else if (JFactory::getApplication()->isSite() && $context == 'com_config.module')
      $context = 'com_modules.module';

    return $context;
  }

  public function onAfterInitialise()
  {
    JLoader::register('n3tfields', __DIR__ . '/helpers/n3tfields.php');
    n3tFields::registerContext('article', 'com_content.article');
    n3tFields::registerContext('category', 'com_content.category');
    n3tFields::registerContext('banner', 'com_banners.banner');
    n3tFields::registerContext('module', 'com_modules.module');
    n3tFields::registerContext('menu', 'com_menus.item');
    n3tFields::registerContext('user', 'com_users.user');
  }

  public function onContentPrepareForm($form, $data)
  {
    if (!($form instanceof JForm)) {
      $this->_subject->setError('JERROR_NOT_A_FORM');
      return false;
    }

    $context = $this->getContext($form->getName());

    $files = $this->getFiles($context);
    $lang = JFactory::getLanguage();

    JLoader::register('FieldsHelper', JPATH_ADMINISTRATOR . '/components/com_fields/helpers/fields.php');
    $fieldTypes = FieldsHelper::getFieldTypes();
    foreach ($fieldTypes as $field) {
      if ($path = $field['path'])
        JFormHelper::addFieldPath($path);

      if ($path = $field['rules'])
        JFormHelper::addRulePath($path);
    }

    foreach ($files as $template => $file) {
      $lang->load('tpl_' . $template , JPATH_SITE . '/templates/' . $template, $lang->getTag(), true);
      $form->loadFile($file, false);
    }

    return true;
  }

  public function onContentPrepareData($context, &$data)
  {
    if (is_array($data) && isset($data['id']))
      $id = $data['id'];
    elseif (is_object($data) && isset($data->id))
      $id = $data->id;
    else
      return true;

    if (empty($id))
      return true;

    $context = $this->getContext($context);

    $db = JFactory::getDbo();
    $query = $db->getQuery(true)
      ->select($db->quoteName('value'))
      ->from('#__n3tfields')
      ->where($db->quoteName('context') . ' = ' . $db->quote($context))
      ->where($db->quoteName('item_id') . ' = ' . $id);
    $db->setQuery($query);

    if (is_array($data)) {
      $data['n3tfields'] = new Registry();
      $data['n3tfields'] = $data['n3tfields']->loadString($db->loadResult());
    } else {
      $data->n3tfields = new Registry();
      $data->n3tfields = $data->n3tfields->loadString($db->loadResult());
    }

    return true;
  }

  public function onContentAfterSave($context, $item, $isNew, $data = array())
  {
    if (empty($data))
      $data = JFactory::getApplication()->input->post->get('jform', array(), 'array');

    if (!is_array($data) || empty($item->id))
      return true;

    if (!isset($data['n3tfields']) || !is_array($data['n3tfields']))
      return true;

    $context = $this->getContext($context);

    $value = new stdClass();
    $value->context = $context;
    $value->item_id = $item->id;
    $value->value = new Registry();
    $value->value->loadArray($data['n3tfields']);
    $value->value = $value->value->toString();

    $db = JFactory::getDbo();
    $query = $db->getQuery(true)
      ->select($db->quoteName('item_id'))
      ->from('#__n3tfields')
      ->where($db->quoteName('context') . ' = ' . $db->quote($context))
      ->where($db->quoteName('item_id') . ' = ' . $item->id);
    $db->setQuery($query);

    if ($db->loadResult())
      $db->updateObject('#__n3tfields', $value, array('context', 'item_id'));
    else
      $db->insertObject('#__n3tfields', $value);

    n3tFields::cleanCache($context, $item);

    return true;
  }

  public function onContentAfterDelete($context, $item)
  {
    if (empty($item->id))
      return true;

    $context = $this->getContext($context);

    $db = JFactory::getDbo();
    $query = $db->getQuery(true)
      ->delete('#__n3tfields')
      ->where($db->quoteName('context') . ' = ' . $db->quote($context))
      ->where($db->quoteName('item_id') . ' = ' . $item->id);
    $db->setQuery($query);
    $db->execute();

    n3tFields::cleanCache($context, $item);

    return true;
  }

  public function onExtensionAfterSave($context, $item, $isNew)
  {
    return $this->onContentAfterSave($context, $item, $isNew, $data);
  }

  public function onExtensionAfterDelete ($context, $item)
  {
    return $this->onContentAfterDelete($context, $item);
  }

  public function onUserAfterSave($user, $isNew, $success, $msg)
  {
    if (!$user['id'] || !$success)
      return true;

    return $this->onContentAfterSave('com_users.user', JFactory::getUser($user['id']), $isNew, $data);
  }

  public function onUserAfterDelete($user, $succes, $msg)
  {
    return $this->onContentAfterDelete('com_users.user', (object)$item);
  }
}
